@@ -16,62 +16,14 @@ layer <-
              groups = NULL,
              style = NULL, force = FALSE,
              theme = if (force) trellis.par.get() else NULL,
-             under = FALSE, superpose = FALSE,
-             ## Deprecated:
-             eval = FALSE, etc = FALSE)
+             under = FALSE, superpose = FALSE)
 {
     ## set layer to quoted expressions in `...`
     foo <- eval(substitute(expression(...)))
-    if (eval) {
-        ## evaluate arguments to calls immediately in the calling environment
-        .Deprecated(msg = "The 'eval' argument of layer() is Deprecated; use 'data'.")
-        for (i in seq_along(foo)) {
-            icall <- foo[[i]]
-            ## allow special names to refer to common internal arguments;
-            ## but this is now Deprecated in favour of the 'etc' argument.
-            icall <- eval(call("substitute",
-                               icall, list(.x = quote(quote(x)),
-                                           .y = quote(quote(y)),
-                                           .z = quote(quote(z)),
-                                           .groups = quote(quote(groups)),
-                                           .subscripts = quote(quote(subscripts)))))
-            if (identical(etc, FALSE)) {
-                icall[-1] <- lapply(icall[-1], eval.parent)
-            } else {
-                if (any(sapply(Args, identical, as.symbol("..."))))
-                    stop("The dots `...` should not be included when etc = TRUE.")
-                ## pass on all un-named arguments from dots
-                Args <- lapply(icall[-1], eval.parent)
-                icall <-
-                    substitute(do.call(.FUN,
-                                       modifyList(list(...)[etc], Args)),
-                               list(.FUN = icall[[1]],
-                                    Args = Args,
-                                    etc = etc))
-            }
-            foo[[i]] <- icall
-        }
-    } else if (!identical(etc, FALSE)) {
-        ## pass on all un-named arguments from dots
-        .Deprecated(msg = "The 'etc' argument of layer() is Deprecated; it happens automatically now.")
-        for (i in seq_along(foo)) {
-            icall <- foo[[i]]
-            Args <- as.list(icall)[-1]
-            if (any(sapply(Args, identical, as.symbol("..."))))
-                stop("The dots `...` should not be included when etc = TRUE.")
-            icall <- substitute(do.call(.FUN,
-                                        modifyList(list(...)[etc], Args)),
-                                list(.FUN = icall[[1]],
-                                    Args = Args,
-                                    etc = etc))
-            foo[[i]] <- icall
-        }
-    } else {
+    if (magicdots) {
         ## The dots `...` are magic:
         ## pass on only those arguments not named in each call
-        if (magicdots) {
-            foo <- as.expression(lapply(foo, magicDots, exclude = exclude))
-        }
+        foo <- as.expression(lapply(foo, magicDots, exclude = exclude))
     }
     mostattributes(foo) <-
         list(data = data,
@@ -92,10 +44,12 @@ layer <-
 ## not named in the call
 magicDots <- function(ocall, exclude = NULL, assume.xy = TRUE)
 {
-    stopifnot(is.call(ocall))
+    if (!is.call(ocall)) stop("arguments to layer() should be calls")
     ## call recursively with any calls inside this one
     for (i in seq_along(ocall)[-1]) {
         thisArg <- ocall[[i]]
+        if (missing(thisArg)) ## eg x[,1]
+            next
         if (is.call(thisArg)) {
             ## skip function definitions
             if (identical(thisArg[[1]], as.symbol("function")))
@@ -164,7 +118,7 @@ glayer_ <- function(...)
 }
 
 ## to avoid print.trellis
-print.layer <- print.default
+print.layer <- function(x, ...) print.default(x, ...)
 
 ## to avoid [.trellis and to keep the class attribute
 "[.layer" <- function (x, i, ...)
@@ -173,6 +127,7 @@ print.layer <- print.default
 "+.trellis" <- function(object, lay)
 {
     ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(`+`)
+    if (missing(object) || missing(lay)) stop("Only one argument supplied to binary operator + which requires two.")
     stopifnot(inherits(object, "trellis"))
     lay <- as.layer(lay)
     if (inherits(object, "layer")) {
@@ -180,7 +135,7 @@ print.layer <- print.default
         return(structure(c(unclass(object), unclass(lay)),
                          class = c("layer", "trellis")))
     }
-    panel <- if (toString(object$call[[1]]) == "splom")
+    panel <- if ("panel" %in% names(object$panel.args.common))
         object$panel.args.common$panel
     else object$panel
     panel <- if (is.function(panel)) panel
@@ -195,35 +150,37 @@ print.layer <- print.default
     ## a flag to indicate this panel function has layers
     ## (used by flattenPanel and undoLayer)
     .is.a.layer <- TRUE
-    newpanel <- function(..., subscripts = TRUE) {
+    newpanel <- function(...) {
         .UNDER <- unlist(lapply(lay, attr, "under"))
-        ## underlayers only
-        drawLayer(lay[.UNDER])
+        ## underlaying items only
+        drawLayer(lay[.UNDER], list(...))
         ## original panel function:
-        panel(..., subscripts = subscripts)
-        ## overlayers only
-        drawLayer(lay[.UNDER == FALSE])
+        panel(...)
+        ## overlaying items only
+        drawLayer(lay[.UNDER == FALSE], list(...))
     }
-    object <- update(object, panel = newpanel)
+    if ("panel" %in% names(object$panel.args.common))
+        object$panel.args.common$panel <- newpanel
+    else object$panel <- newpanel
     ## need this to allow further calls to update() to insert arguments:
     object$call <- call("update", ocall)
     object
 }
 
-drawLayer <- function(lay)
+drawLayer <- function(lay, panelArgs = trellis.panelArgs())
 {
     lay <- as.layer(lay)
     .UNDER <- unlist(lapply(lay, attr, "under"))
     ## underlayers, in reverse order
     for (.ITEM in rev(lay[.UNDER]))
-        drawLayerItem(.ITEM)
+        drawLayerItem(.ITEM, panelArgs)
     ## overlayers
     for (.ITEM in lay[.UNDER == FALSE])
-        drawLayerItem(.ITEM)
+        drawLayerItem(.ITEM, panelArgs)
     invisible()
 }
 
-drawLayerItem <- function(layer.item)
+drawLayerItem <- function(layer.item, panelArgs)
 {
     stopifnot(is.expression(layer.item))
     ## check that any restrictions on packets/rows/columns are met
@@ -260,15 +217,23 @@ drawLayerItem <- function(layer.item)
         if (!is.null(attr(layer.item, "style"))) {
             ## extract plot style attributes from given index into superpose.*
             .TRELLISPAR <- trellis.par.get()
-            .STY <- attr(layer.item, "style")
-            trellis.par.set(plot.line = Rows(trellis.par.get("superpose.line"), .STY),
-                            add.line = Rows(trellis.par.get("superpose.line"), .STY),
-                            add.text = Rows(trellis.par.get("superpose.line"), .STY),
-                            plot.symbol = Rows(trellis.par.get("superpose.symbol"), .STY),
-                            plot.polygon = Rows(trellis.par.get("superpose.polygon"), .STY),
-                            axis.text = Rows(trellis.par.get("superpose.line"), .STY),
-                            axis.line = Rows(trellis.par.get("superpose.line"), .STY)
-                            )
+            local({
+                i <- attr(layer.item, "style")
+                line <- Rows(trellis.par.get("superpose.line"), i)
+                symbol <- Rows(trellis.par.get("superpose.symbol"), i)
+                polygon <- Rows(trellis.par.get("superpose.polygon"), i)
+                trellis.par.set(plot.line = line,
+                                superpose.line = line,
+                                add.line = line,
+                                add.text = line,
+                                plot.symbol = symbol,
+                                superpose.symbol = symbol,
+                                plot.polygon = polygon,
+                                superpose.polygon = polygon,
+                                axis.text = line,
+                                axis.line = line
+                                )
+            })
             on.exit(trellis.par.set(.TRELLISPAR))
         }
         with(dots,
@@ -278,10 +243,10 @@ drawLayerItem <- function(layer.item)
     ## call panel.superpose for group layers
     if (isTRUE(attr(layer.item, "superpose"))) {
         do.call("panel.superpose",
-                modifyList(trellis.panelArgs(),
+                modifyList(panelArgs,
                   list(panel.groups = drawLayerItemPerGroup)))
     } else {
-        do.call("drawLayerItemPerGroup", trellis.panelArgs())
+        do.call("drawLayerItemPerGroup", panelArgs)
     }
 }
 
